@@ -1,8 +1,4 @@
-//
-// Sysinfo
-//
-// Copyright (c) 2021 Guillaume Gomez
-//
+// Take a look at the license at the top of the repository in the LICENSE file.
 
 use std::path::Path;
 
@@ -13,12 +9,12 @@ use crate::{DiskUsage, Pid, ProcessExt, ProcessStatus, Signal};
 pub struct Process;
 
 impl ProcessExt for Process {
-    fn new(_pid: Pid, _parent: Option<Pid>, _start_time: u64) -> Process {
-        Process {}
+    fn kill(&self) -> bool {
+        false
     }
 
-    fn kill(&self, _signal: Signal) -> bool {
-        false
+    fn kill_with(&self, _signal: Signal) -> Option<bool> {
+        None
     }
 
     fn name(&self) -> &str {
@@ -69,6 +65,10 @@ impl ProcessExt for Process {
         0
     }
 
+    fn run_time(&self) -> u64 {
+        0
+    }
+
     fn cpu_usage(&self) -> f32 {
         0.0
     }
