@@ -390,8 +390,43 @@ impl System {
     }
 }
 
-// FIXME: to be removed once 0.2.108 libc has been published!
-const CPUSTATES: usize = 5;
+#[derive(Debug)]
+struct Zfs {
+    enabled: bool,
+    mib_arcstats_size: [c_int; 5],
+}
+
+impl Zfs {
+    fn new() -> Self {
+        let mut zfs = Self {
+            enabled: false,
+            mib_arcstats_size: Default::default(),
+        };
+        unsafe {
+            init_mib(
+                b"kstat.zfs.misc.arcstats.size\0",
+                &mut zfs.mib_arcstats_size,
+            );
+            let mut arc_size: u64 = 0;
+            if get_sys_value(&zfs.mib_arcstats_size, &mut arc_size) {
+                zfs.enabled = arc_size != 0;
+            }
+        }
+        zfs
+    }
+
+    fn arc_size(&self) -> Option<u64> {
+        if self.enabled {
+            let mut arc_size: u64 = 0;
+            unsafe {
+                get_sys_value(&self.mib_arcstats_size, &mut arc_size);
+            }
+            Some(arc_size)
+        } else {
+            None
+        }
+    }
+}
 
 /// This struct is used to get system information more easily.
 #[derive(Debug)]
@@ -422,6 +457,7 @@ struct SystemInfo {
     /// processes' CPU usage.
     fscale: f32,
     procstat: *mut libc::procstat,
+    zfs: Zfs,
 }
 
 // This is needed because `kd: *mut libc::kvm_t` isn't thread-safe.
@@ -475,10 +511,11 @@ impl SystemInfo {
             kd,
             mib_cp_time: Default::default(),
             mib_cp_times: Default::default(),
-            cp_time: utils::VecSwitcher::new(vec![0; CPUSTATES]),
-            cp_times: utils::VecSwitcher::new(vec![0; nb_cpus as usize * CPUSTATES]),
+            cp_time: utils::VecSwitcher::new(vec![0; libc::CPUSTATES as usize]),
+            cp_times: utils::VecSwitcher::new(vec![0; nb_cpus as usize * libc::CPUSTATES as usize]),
             fscale: 0.,
             procstat: std::ptr::null_mut(),
+            zfs: Zfs::new(),
         };
         unsafe {
             let mut fscale: c_int = 0;
@@ -543,8 +580,8 @@ impl SystemInfo {
 
     /// Returns (used, total).
     fn get_swap_info(&self) -> (u64, u64) {
-        // Magic number used in htop. Cannot find how they got when reading `kvm_getswapinfo` source
-        // code so here we go...
+        // Magic number used in htop. Cannot find how they got it when reading `kvm_getswapinfo`
+        // source code so here we go...
         const LEN: usize = 16;
         let mut swap = MaybeUninit::<[libc::kvm_swap; LEN]>::uninit();
         unsafe {
@@ -592,7 +629,13 @@ impl SystemInfo {
             get_sys_value(&self.virtual_wire_count, &mut mem_wire);
         }
 
-        let used = (mem_active * self.page_size as u64) + (mem_wire * self.page_size as u64);
+        let mut mem_wire = mem_wire * self.page_size as u64;
+        // We need to subtract "ZFS ARC" from the "wired memory" because it should belongs to cache
+        // but the kernel reports it as "wired memory" instead...
+        if let Some(arc_size) = self.zfs.arc_size() {
+            mem_wire -= arc_size;
+        }
+        let used = (mem_active * self.page_size as u64) + mem_wire;
         used / 1_000
     }
 
@@ -631,7 +674,7 @@ impl SystemInfo {
             let mut total_old: u64 = 0;
             let mut cp_diff: libc::c_ulong = 0;
 
-            for i in 0..(CPUSTATES as usize) {
+            for i in 0..(libc::CPUSTATES as usize) {
                 // We obviously don't want to get the idle part of the processor usage, otherwise
                 // we would always be at 100%...
                 if i != libc::CP_IDLE as usize {
@@ -653,7 +696,7 @@ impl SystemInfo {
         let old_cp_times = self.cp_times.get_old();
         let new_cp_times = self.cp_times.get_new();
         for (pos, proc_) in processors.iter_mut().enumerate() {
-            let index = pos * CPUSTATES as usize;
+            let index = pos * libc::CPUSTATES as usize;
 
             fill_processor(proc_, &new_cp_times[index..], &old_cp_times[index..]);
         }
